#!/usr/bin/perl -w

use strict 'vars';

# A Nagios check that uses arkc, the command line control utility for
# the Arkeia Network Backup program. It checks whether or not a backup
# is running at the moment or if one is waiting for a new tape. "No backup
# is running" is considered the "Ok" state.
#
# Written by Moritz Bunkus <m.bunkus@linet-services.de>
# You are free to do whatever you want with this script.
#
# You have to configure arkc first. The user running check_arkeia.pl
# must create a directory ~/.arkc and a file called arkc.param inside
# that directory. The arkc.param file must contain these five lines:
#
# NODE "name.of.your.backup.server"
# LOGIN "your_username"
# PASSWORD "the_password"
# LANG="EN"
# PORTNUM "617"
#
# Replace the dummy values with your real ones, of course. It's also a
# good idea to 'chmod og-rwx ~/.arkc/arkc.param' so that others won't
# be able to view your password.
#
# This script takes no parameters. If exits with "Ok" if no backup is
# running at the moment.
#
# If exactly one backup is running at the moment then it exits with
# "Warning".
#
# In all the other situations (more than one backup running or exactly
# one backup running, but that one is waiting for a new tape) it will
# exit with an "Error" and display the number of running backups.


# You may have to add the full path to the arkc utility here.
my $arkc = "arkc";

my (@output, $num_running, $num_total, $num_waiting_for_tape, $line);

@output = `$arkc -backup -running`;

if ($? != 0) {
  print("Unknown: '$arkc -backup -running' failed\n");
  exit(2);
}

($num_running, $num_total, $num_waiting_for_tape) = (0, 0, 0);

foreach $line (@output) {
  chomp($line);
  if ($line =~ /bksid=(.*)/) {
    $num_total += 1;
    system("$arkc -backup -status -D bksid=$1 > /dev/null 2> /dev/null");
    if ($? == 0) {
      $num_running += 1;
    } elsif ($? == (1 << 8)) {
      $num_waiting_for_tape += 1;
    } else {
      print("EC: $?\n");
    }
  }
}

if (0 == $num_total) {
  print("Ok: No backup is running\n");
  exit(0);
}

if ((1 == $num_total) && (1 == $num_running)) {
  print("Warning: 1 backup is running\n");
  exit(1);
}

printf("Critical: %d backup%s total (%d running, %d waiting for a tape)\n",
       $num_total, $num_total == 1 ? "" : "s",
       $num_running, $num_waiting_for_tape);
exit(2);
