#!/usr/bin/perl
# -----------------------------------------------

=head1 NAME

check_pcmeasure.pl - Nagios Plugin for pcmeasure (www.messpc.de)

=head1 SYNOPSIS

check_pcmeasure.pl -H <host> -S <sensor> [-p <port>] 
   [-w <min-warn>] [-c <min-crit>] [-v] [-V]
   [-W <max-warn>] [-C <max-crit>] [-R <rrd file>]
  

Connects to the program pcmeasure or a ethernet messbox
to ask for values of connected hardware
sensors. pcmeasure is available as Linux or Windows program, the sensors
are connected via serial or parallel port. On the standalone ethernet
messbox the sensors are connected directly. There are different sensors 
available: temperature, humidity, voltage, smoke, motion, water and more. 
Please check www.messpc.de (german) or wwww.pcmeasure.com (english) 
for further informations.

=head1 OPTIONS

=over 4

=item -H|--host=<name-or-ip>

Hostname or ip address of the server running pcmeasure

=item -p|--port=<port>

The port where the program "pcmeasure" is listening

=item -W|--warn-max=<warn>

Return status "WARN" if value > warn

=item -C|--crit-max=<crit>

Return status "CRITICAL" if value > crit

=item -w|--warn-min=<warn>

Return status "WARN" if value < warn

=item -c|--crit-min=<crit>

Return status "CRITICAL" if value < crit

=item -S|--sensor=<sensor>

The sensor you want to check. 

Examples: com1.1 (sensor 1 on serial port 1)
lpt1.2 (sensor 2 on parallel port 1).

=item -R|--rrd-database=<path-to-db>

You can store the retrieving value in a Round Robin Database RRD.
The database is updated with "N:<value>" (means: current time:given value)
and should be writeable by your nagios user. The rrd database would be
created, if not exist. To change the parameters for database creation,
create rrd database manually or change the values in this plugin
(subroutine create_rrd). 

For infos about Round Robin Databases and the excellent RRDtool please see
www.rrdtool.org. The step parameter of the database should be aligned to
your check interval used with nagios.

=item -v|--verbose

increases verbosity

=item -V|--version

print version an exit

=item -h|--help

print help message and exit

=cut

# -----------------------------------------------

use strict;
use warnings;

use Getopt::Long qw(:config no_ignore_case bundling);
use Pod::Usage;
use IO::Socket::INET;
use RRDs;

use vars qw(%ERRORS);

# -- hard coded error values to avoid path problems with utils.pm
%ERRORS = (
   'OK'       => 0,
   'WARNING'  => 1,
   'ERROR'    => 2,
   'CRITICAL' => 2,
   'UNKNOWN'  => 3,
);

# -----------------------------------------------
# global vars
# -----------------------------------------------

my $TIMEOUT = 10.0;
my $perfdatastring = '';# for performance data
my $ERR;		# RRD error string

# -----------------------------------------------
# Command line Parameters
# -----------------------------------------------

# -- Vars

my $remote_host	= '';	# server running pcmeasure
my $remote_port	= 4000;	# default tcp port
my $warn_max;		#
my $crit_max;		#
my $warn_min;		#
my $crit_min;		#
my $sensor	= '';	# i.e. com1.1, lpt2.4
my $verbose    	= 0;	#
my $help	= 0;	# 
my $rrddb;		# rrd database: store values in an round robin
			# archive, if given
my $printversion = 0;	#

# -- -- -- -- -- -- -- -- 
my $version	= '$Revision: 1.2 $ / $Date: 2013/08/16 15:42:00 $ / wob ';
# -- -- -- -- -- -- -- --

# -- GetOpt

GetOptions(
   "H|host=s"		=> \$remote_host,
   "p|port=s"		=> \$remote_port,
   "W|warn-max=s"	=> \$warn_max,
   "C|crit-max=s"	=> \$crit_max,
   "w|warn-min=s"	=> \$warn_min,
   "c|crit-min=s"	=> \$crit_min,
   "S|sensor=s"		=> \$sensor,
   "R|rrd-database=s"	=> \$rrddb,
   "h|help"		=> \$help,
   "V|version"		=> \$printversion,
   "v|verbose+"		=> \$verbose,

   ) or pod2usage({ -exitval => $ERRORS{'UNKNOWN'},
   		    -verbose => 1,
   	            -msg     => "\n *** unknown argument found ***" });

# -- help message
pod2usage(-verbose => 2,
	  -exitval => $ERRORS{'UNKNOWN'} ,
	 ) if ( $help );

# -- version

pod2usage(-msg     => "\n$0 -- version: $version\n",
          -verbose => 0,
	  -exitval => $ERRORS{'UNKNOWN'} ,
	 ) if ( $printversion );

# -- no host specified
pod2usage(-msg     => "\n*** please specify one host or ip ***\n",
          -verbose => 1,
	  -exitval => $ERRORS{'UNKNOWN'} ,
	 ) if ( $remote_host eq '' );

# -- no sensor specified
pod2usage(-msg     => "\n*** please specify one sensor ***\n",
          -verbose => 1,
	  -exitval => $ERRORS{'UNKNOWN'} ,
	 ) if ( $sensor eq '' );

# -----------------------------------------------
# create/check rrd database file
# -----------------------------------------------
if ( defined($rrddb) ) {
   # -- create database, if not exist
   if ( ! -e $rrddb ) {
      $ERR = &create_rrd($rrddb);
      if ( $ERR ) {
	 print STDOUT "UNKNOWN: can't create RRD database: $ERR\n";
	 exit $ERRORS{'UNKNOWN'};
      }
   }
   # -- check, if database file is writable
   if ( ! -w $rrddb ) {
      print STDOUT "UNKNOWN: RRD database is not writable\n";
      exit $ERRORS{'UNKNOWN'};
   }
}

# -----------------------------------------------
# socket operation
# -----------------------------------------------

my $socket = IO::Socket::INET->new(PeerAddr => $remote_host,
				   PeerPort => $remote_port,
				   Proto    => "tcp",
				   Type     => SOCK_STREAM,
				   Timeout  => $TIMEOUT,
				  );

if ($@) {
   print STDERR "Can't open Socket to host $remote_host\n";
   exit $ERRORS{'UNKNOWN'};
};

print $socket "pcmeasure.$sensor\r\n";
my $answer = <$socket>;
close($socket);

$answer =~ s/[\r][\n]//;

if ($verbose) {
   print STDERR "$answer\n";
}

if ( $answer =~ m/valid=(\d+)/) {
   my $valid = $1;
   if ($valid != 1 ) {
      print STDOUT "WARNING: answer not valid: $answer\n";
      exit $ERRORS{'WARNING'};
   }
} else {
   print STDOUT "UNKNOWN: no valid tag found: $answer\n";
   exit $ERRORS{'UNKNOWN'};
}
   
if ( $answer =~ m/value=\s*([+-]{0,1}[.\d]+)/) {
   my $value = $1;

   if ( defined($rrddb) ) {
      # -- update database	
      my $string = "N:$value";
      RRDs::update( $rrddb, $string );
      $ERR = RRDs::error;
      if ( $ERR ) { 
         print STDOUT "CRITICAL: can't update RRD database: $ERR\n";
         exit $ERRORS{'CRITICAL'};
      }
   }
   # -- performance data
   my $perfcrit = 0;
   my $perfwarn = 0;
 
   # -- using upper limits for perfdata, can only use 
   #    one crit and one warn level
   $perfcrit = $crit_max if (defined($crit_max));
   $perfwarn = $warn_max if (defined($warn_max));

   $perfdatastring = "|value=$value;$perfwarn;$perfcrit;";

   # -- upper limits
   if ( defined($crit_max) && $value > $crit_max ) {
      print STDOUT "Value $sensor: /$value/ > $crit_max $perfdatastring\n";
      exit $ERRORS{'CRITICAL'};
   }
   if ( defined($warn_max) && $value > $warn_max ) {
      print STDOUT "Value $sensor: /$value/ > $warn_max $perfdatastring\n";
      exit $ERRORS{'WARNING'};
   }

   # -- lower limits
   if ( defined($crit_min) && $value < $crit_min ) {
      print STDOUT "Value $sensor: /$value/ < $crit_min $perfdatastring\n";
      exit $ERRORS{'CRITICAL'};
   }
   if ( defined($warn_min) && $value < $warn_min ) {
      print STDOUT "Value $sensor: /$value/ < $warn_min $perfdatastring\n";
      exit $ERRORS{'WARNING'};
   }

   # -- no limit, OK

   print STDOUT "Value $sensor: /$value/ $perfdatastring\n";
   exit $ERRORS{'OK'};

} else {
   print STDOUT "UNKNOWN: $answer\n";
   exit $ERRORS{'UNKNOWN'};
}

# ===============================================

=head1 RRD DATABASE CREATION

The step of an automatically created rrd database is 120 seconds, the
heartbeat 600 with xfiles factor 0.5. This means: the time resolution is
120 seconds, undependent of the used period for measurement. There must be
at least one valid value within 300 seconds ( xfiles factor 0.5 x
heartbeat 600).

All values for database creation:

 "--step=120",                   # 120 seconds
 "--start=-1d",                  # start one day back
 "DS:value:GAUGE:600:U:U",       # heartbeat
 "RRA:AVERAGE:0.5:1:3600",       # 3600 x 1    x step (120s) = 120h = 5d
 "RRA:AVERAGE:0.5:30:700",       # 700  x 30   x step        = 700h = 29d
 "RRA:AVERAGE:0.5:120:775",      # 775  x 120  x step        = 3100h = 129d
 "RRA:AVERAGE:0.5:1440:797",     # 797  x 1440 x step        = 1594d
 "RRA:MAX:0.5:1:3600",
 "RRA:MAX:0.5:30:700",
 "RRA:MAX:0.5:120:775",
 "RRA:MAX:0.5:1440:797",
 "RRA:MIN:0.5:1:3600",
 "RRA:MIN:0.5:30:700",
 "RRA:MIN:0.5:120:775",
 "RRA:MIN:0.5:1440:797",

To change this values, you can edit this plugin or create the rrd database
manually.

=cut

# ---
sub create_rrd{
   my $file = shift;
   RRDs::create( $rrddb,
        "--step=120",			# 120 seconds
        "--start=-1d",			# start one day back
        "DS:value:GAUGE:600:U:U",	# heartbeat
        "RRA:AVERAGE:0.5:1:3600",	# 3600 x 1    x step (120s) = 120h = 5d
        "RRA:AVERAGE:0.5:30:700",	# 700  x 30   x step        = 700h = 29d
        "RRA:AVERAGE:0.5:120:775",	# 775  x 120  x step        = 3100h = 129d
        "RRA:AVERAGE:0.5:1440:797",	# 797  x 1440 x step        = 1594d
        "RRA:MAX:0.5:1:3600",
        "RRA:MAX:0.5:30:700",
        "RRA:MAX:0.5:120:775",
        "RRA:MAX:0.5:1440:797",
        "RRA:MIN:0.5:1:3600",
        "RRA:MIN:0.5:30:700",
        "RRA:MIN:0.5:120:775",
        "RRA:MIN:0.5:1440:797",
   );
   my $ERR = RRDs::error;
   return $ERR;
}

__END__
# ===============================================

=head1 AUTHOR

wob (at) swobspace (dot) net

=head1 KNOWN ISSUES

You can only check one sensor per call.

Nagios threshold range format not yet implemented.

=head1 BUGS

may be

=head1 LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License (and no
later version).

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

=head1 HISTORY
 
$Log: check_pcmeasure.pl,v $
Revision 1.2  2013/08/16 15:42:00  rbe
*** empty log message ***

Revision 1.1  2013/08/16 15:40:38  rbe
Initial revision


Revision 1.7  2006/06/09 16:42:00  wob

 - Bugfix: add missing ERRORS{'CRITICAL'} value

Revision 1.6  2006/05/21 10:49:02  wob

 - hard coded error values to avoid path problems calling utils.pm
 - output contains now performance data
 - automatic rrd database creation, if database did not exist


Revision 1.5  2005/06/29 13:55:19  wob
History (CVS) and Revision Information inserted

